classdef DCProblemAsGSIP < ImplicitGSIPProblem
    %p=DCProblemAsGSIP(d,c) creates an instance of a design-centering
    %problem out of the design d and the container c 
    
    properties (GetAccess=public)
        container
        design
    end

    properties (GetAccess=protected)
        %upper level
        nrOfUppLevVars
        nrOfSemiInfConstrs
        
        boundsOnUppLevVars
        
        initValsOfUppLevVars
        
        nrOfNonlinIneqsUppLev
        nrOfNonlinEqsUppLev
        
        linEqsOnUppLevVars
        linIneqsOnUppLevVars
        
        %lower level        
        nrOfLowLevVars
        nrOfLowLevNonlinConstrs
        
        
        %problem assignment
        con2LowLev;
        nrOfIndexset=1;
    end
    
    methods
        function p= DCProblemAsGSIP(Design, Container)
                  
            
            
            
            
            %check that the design and the container have the same
            %dimension
            assert(Design.getDim()==Container.getDim(), 'The dimensions of the design and the container does not coincide');
            
            %set the design and the container
            p.design=Design;
            p.container= Container;
            
            %bounds on upper level variables
            [lb,ub]=Design.getBoundsOnParams;
            p.boundsOnUppLevVars=[lb;ub];

            %set the linear constraints and the number of non-linear
            %constraints
            [A,b]=Design.getLinEqsOnParams();            
            p.linEqsOnUppLevVars = [A,b];
            [A,b]=Design.getLinIneqsOnParams();
            p.linIneqsOnUppLevVars = [A,b];
            p.nrOfNonlinIneqsUppLev=Design.getNrNonlinIneqConstrsOnParams();
            p.nrOfNonlinEqsUppLev=Design.getNrNonlinEqConstrsOnParams();
            
            %set the dimension of the Variables and the number of
            %constraints
            p.nrOfUppLevVars=Design.getNrParams();
            p.nrOfSemiInfConstrs=Container.getNrDescrFuns();
            p.nrOfLowLevVars =Container.getDim();
            p.nrOfLowLevNonlinConstrs=Design.getNrNonlinDescrFuns();
            
            
            %set initial values
            p.initValsOfUppLevVars=Design.getInitValsParams();
            
            %set the problem assignment
            p.con2LowLev=ones(1,p.nrOfSemiInfConstrs);
            
            %read out more properties
        end
        
        %objective function (negative volume)
        function f=evalObjFun(obj,x)                
            f=-obj.design.vol(x);
        end         
        function Dx_f=DxObjFun(obj,x)
            Dx_f=-obj.design.DxVol(x);
        end
        function grad_f=gradObjFun(obj,x)
            grad_f=-obj.design.gradVol(x);
        end
        function DxDxf=DxDxObjFun(obj,x)
            DxDxf=-obj.design.DxDxVol(x);            
        end
        
        %semi-infinite constraints (Containment funciton of the container)
        function g=evalSemiInfConstrs(obj,i,~,y) 
            g=obj.container.evalDescrFuns(i,y);            
        end
        function Dx_g=DxSemiInfConstrs(obj,~,~,y)
            m=size(y,1);
            Dx_g=zeros(m,obj.nrOfUppLevVars);
        end
        function Dy_g=DySemiInfConstrs(obj,i,~,y)
            Dy_g=obj.container.DyDescrFuns(i,y);
        end
        function DxDx_g=DxDxSemiInfConstrs(obj,i,~,y)
            DxDx_g=zeros(obj.nrOfUppLevVars);
        end
        function DxDy_g=DxDySemiInfConstrs(obj,i,~,y)
            DxDy_g=zeros(obj.nrOfLowLevVars,obj.nrOfUppLevVars);
        end
        function DyDy_g=DyDySemiInfConstrs(obj,i,~,y)
            DyDy_g=obj.container.DyDyDescrFuns(i,y);
        end
        
        
        
        %More non-linear constraints on the variables (constraints on the
        %design parameters)
        function k=evalUppLevNonlinIneqConstrs(obj,x)
            k=obj.design.evalNonlinIneqConstrsOnParams(x);
        end
        function k=evalUppLevNonlinEqConstrs(obj,x)
            k=obj.design.evalNonlinEqConstrsOnParams(x);
        end
        function Dx_k=DxUppLevNonlinIneqConstrs(obj,x)
            Dx_k=obj.design.DxNonlinIneqConstrsOnParams(x); 
        end
        function Dx_k=DxUppLevNonlinEqConstrs(obj,x)
            Dx_k=obj.design.DxNonlinEqConstrsOnParams(x);
        end  
        function DxDx_k=DxDxUppLevNonlinIneqConstrs(obj,x)
            DxDx_k=obj.design.DxDxNonlinIneqConstrsOnParams(x); 
        end
        function DxDx_k=DxDxUppLevNonlinEqConstrs(obj,x)
            DxDx_k=obj.design.DxDxNonlinEqConstrsOnParams(x);
        end  
        
        % lower level functions (design functions)
        function u=evalLowLevNonlinConstrs(obj,~,x,y)
            u=obj.design.evalNonlinDescrFuns(x,y);
        end
        function Dy_u=DyLowLevNonlinConstrs(obj,~,x,y)
            Dy_u=obj.design.DyNonlinDescrFuns(x,y);
        end
        function Dx_u=DxLowLevNonlinConstrs(obj,~,x,y)
            Dx_u=obj.design.DxNonlinDescrFuns(x,y);
        end
        function DxDy_u=DxDyLowLevNonlinConstrs(obj,~,x,y)
            DxDy_u=obj.design.DxDyNonlinDescrFuns(x,y);
        end
        function DxDx_u=DxDxLowLevNonlinConstrs(obj,~,x,y)
            DxDx_u=obj.design.DxDxNonlinDescrFuns(x,y);
        end
        function DyDy_u=DyDyLowLevNonlinConstrs(obj,~,x,y)
            DyDy_u=obj.design.DyDyNonlinDescrFuns(x,y);
        end
        
        
        function [A,b]=getLinIneqsOnLowLevVars(obj,i,x)
            [A,b]=obj.design.getLinDescrFuns(x);
        end
        function [DxA,Dxb]=DxLinIneqsOnLowLevVars(obj,i,x)
            [DxA,Dxb]=obj.design.DxLinDescrFuns(x);
        end
        function [DxDxA,DxDxb]=DxDxLinIneqsOnLowLevVars(obj,i,x)
            [DxDxA,DxDxb]=obj.design.DxDxLinDescrFuns(x);
        end
        
        %starting point for the optimization of the lower level (inner
        %point)
        function y=getInitValsLowLevVars(obj,~,x)
            y=obj.design.getInteriorPoint(x); 
        end
        function Dx_y=DxStartingPointLowLev(obj,~,x)  
            Dx_y=obj.design.DxInnerPoint(x);
        end         
        
        %Bounds for the lower level Vars (bounding box of container and
        %design)
        function [lb,ub]=getBoundsOnLowLevVars(obj,j,x)
            [lb,ub]=obj.design.getBoundingBox(x);
        end
        function [Dxlb,Dxub]=DxBoundsOnLowLevVars(obj,j,x)
            [Dxlb,Dxub]=obj.design.DxBoundingBox(x);
        end
        function [DxDxlb,DxDxub]=DxDxBoundsOnLowLevVars(obj,j,x)
            [DxDxlb,DxDxub]=obj.design.DxDxBoundingBox(x);
        end        
        function [lb,ub]=getLowLevBoundingBox(obj,j,x)
            [lb,ub]=obj.getBoundsOnLowLevVars(j,x);
        end
        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
            y=obj.design.evalTrafoFun(x,z);
       end
        function z=evalInvTrafoFun(obj,i,x,y)
            z=obj.design.evalInvTrafoFun(x,y);
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
            Dx_y=obj.design.DxTrafoFun(x,z);
        end
        function Dz_y=DzTrafoFun(obj,i,x,z)
            Dz_y=obj.design.DzTrafoFun(x,z);
        end
        function DxDx_y=DxDxTrafoFun(obj,i,x,z)
            DxDx_y=obj.design.DxDxTrafoFun(x,z);
        end
        function DxDz_y=DxDzTrafoFun(obj,i,x,z)
            DxDz_y=obj.design.DxDzTrafoFun(x,z);
        end
        function DzDz_y=DzDzTrafoFun(obj,i,x,z)
            DzDz_y=obj.design.DzDzTrafoFun(x,z);
        end
        function detDz_y=detDzTrafoFun(obj,i,x,z)
            detDz_y=obj.design.detDzTrafoFun(x,z);
        end
        function [lb,ub]=getBoxZ(obj,i)
            [lb,ub]=obj.design.getBoxZ();
        end
        
    end
    
    
    %plotFunction
    methods
        function []=plotSols(obj,x0,varargin)
            switch obj.nrOfLowLevVars 
                case 2
                    obj.plotSolution2D(x0,varargin{:})
                case 3
                    obj.plotSolution3D(x0,varargin{:})
                otherwise
                    warning('There is no routine "plotSols" for a dimension higher than 3'); 
            end
            
        end
    end
    
   
    
    
end

