function [hf] = plotGSIPContainmentCondition( obj,indexOfLowLev,x,lb,ub,varargin)
%plots the lower level problem of an GSIP given with a implicit description
%of the lower level.
%
%   [hf]=<problemname>.plotContainmentCondition(indexOfLowLev,x,lb,ub,
%                   points1,color1,points2,color2,...) 
%   'x' is a 1xm vector, whicht represents the actual parameter. 'lb' are 
%   the lowerbounds of the plot and 'ub' are the upper bounds. If 'lb' 
%   ('ub') is a row-vector, the bounding vector is used for every lower
%   level problem. If  'lb' ('ub') is a matrix the i-th row give the bounds
%   for the i-th problem. The third possibility is to give a cell of
%   row-vectors. The i'th cell give the bounds for the i-th problem.
%   The inputs pointsi and colori give the possibility to mark special
%   points. The input pointsi is either a matrix or a cell of matrices. If 
%   it is a matrix, every row gives one point for every lower level
%   problem. If pointsi is a cell the j-th matrix represents all points
%   that shall be plotted in the picture of the j-th lower level problem.
%   The corresponding input colori is a matlab specifier of a color or a
%   cell of it. If it is only one specifier, the color is taken in every
%   lower level, otherwise the j-th one is take for the j-th lower level.
%
%   [hf]=<problemname>.plotLowerLevelOfGSIPImplicitly(indexOfLowLev,x,lb,ub,
%                   points1,color1,points2,color2,...'filled') 
%   The same as above. Only the two dimensional plot is filled instead of
%   only contour.


%maximal size of the plot
maxwidth=50;
maxheight=30;



%%check wether the given parameters have the correct size

%check the indices of lower levels
assert(isa(indexOfLowLev,'double'),'Please insert a vector for ''indexOfLowLev''');
indexOfLowLev=reshape(indexOfLowLev,1,size(indexOfLowLev,1)*size(indexOfLowLev,2));

%%Globally set number of index sets
NrIndexSets=size(indexOfLowLev,2);


for i=1:NrIndexSets
  assert(~mod(indexOfLowLev(i),1),'please insert indices for ''indexOfLowLev''')  
  assert(indexOfLowLev(i)>0&&indexOfLowLev(i)<=obj.getNrIndexSets,'At least one inserted index in''indexOfLowLev'' is not a valid')
end

%check the parameters
if size(x,1)>1
    x=x';
end
assert(size(x,1)==1,'The parameter ''x'' must be a vector')
assert(size(x,2)==obj.getNrUppLevVars,'The dimension of ''x'' must be the same as the number of upper level vars')

%check the lower bounds and bring them into the right form
%lower bounds can be a vector, a matrix or a cell
%check wether all indexsets must habe the same dimension
sameDimEveryIndex=false;

assert(size(lb,2)==obj.getNrLowLevVars(indexOfLowLev(1)),'The number of columns of ''lb'' must be the same as the number of lower level vars')
if isa(lb,'double')
    sameDimEveryIndex=true;
    if size(lb,1)==1
        assert(min(lb~=-inf)==1,'the bounds  ''lb'' must contain values not equal to -inf')
        lb_temp=lb;
        lb=cell(1,NrIndexSets);
        for i=1:NrIndexSets
            lb{i}=lb_temp;
        end
    else
        assert(size(lb,1)==NrIndexSets,'The number of rows of lb is not the same as the given number of index sets')
        assert(min(min(lb~=-inf))==1,'the bounds  ''lb'' must contain values not equal to -inf')
        lb_temp=lb;
        lb=cell(1,NrIndexSets);
        for i=1:NrIndexSets
            lb{i}=lb_temp(i,:);
        end
    end  
else
    assert(size(lb,1)==NrIndexSets,'The length of lb is not the same as the given number of index sets')
    for i=1:NrIndexSets
        assert(min(lb{i}~=-inf)==1,'the bounds  ''lb'' must contain values not equal to -inf')
    end
end
%check the upper bounds and bring them into the right form
%upper bounds can be a vector a matrix or a cell
assert(size(ub,2)==obj.getNrLowLevVars(indexOfLowLev(1)),'The number of columns of ''ub'' must be the same as the number of lower level vars')
if isa(ub,'double')
    sameDimEveryIndex=true;
    if size(ub,1)==1
        assert(min(ub~=inf)==1,'the bounds  ''ub'' must contain values not equal to inf')
        ub_temp=ub;
        ub=cell(1,NrIndexSets);
        for i=1:NrIndexSets
            ub{i}=ub_temp;
        end
    else
        assert(size(ub,1)==NrIndexSets,'The number of rows of ub is not the same as the given number of index sets')
        assert(min(min(ub~=inf))==1,'the bounds  ''ub'' must contain values not equal to inf')
        ub_temp=ub;
        ub=cell(1,NrIndexSets);
        for i=1:NrIndexSets
            ub{i}=ub_temp(i,:);
        end
    end
else
    assert(length(ub)==NrIndexSets,'The length of ub is not the same as the given number of index sets')
    for i=1:NrIndexSets
        assert(min(ub{i}~=inf)==1,'the bounds  ''ub'' must contain values not equal to inf')
    end
end

%check the points
nrpoints=floor((nargin-5)/2);
for i=1:nrpoints
   Y=varargin{2*i-1};
   
    if isa(Y,'cell')
         assert(length(Y)==NrIndexSets,'The number of given index sets is not equal to the number of cells in the given points')
         for j=1:NrIndexSets
             assert(size(Y{j},2)==obj.getNrLowLevVars(indexOfLowLev(j)),'Each cell of ''Points'' must be a matrix with each row is a vector of a point with dimension of the lower level') 
             for k=1:size(Y{j},1)
                 if max((lb{j}>Y{j}(k,:)))||max((Y{j}(k,:)>ub{j}))
                     warning(sprintf('The point number %i of the %i-th cell of Points%i does not lie inside of the bounds',k,j,i));
                 end
             end
         end
    else
        sameDimEveryIndex=true;
        if size(Y,1)~=0             
             assert(size(Y,2)==obj.getNrLowLevVars(indexOfLowLev(1)),'''Points'' must be a matrix with each row is a vector of a point with dimension of the lower level') 
             for j=1:NrIndexSets
                 for k=1:size(Y,1)
                     if max((lb{j}>Y(k,:)))||max((Y(k,:)>ub{j}))
                         warning(sprintf('The point number %i of Points%i does not lie inside of the bounds',k,i));
                     end
                 end
             end
         end
    end
end
 

%check if all indexsets have the same dimension if necessary
if sameDimEveryIndex
    for i=2:1:NrIndexSets
        assert(obj.getNrLowLevVars(indexOfLowLev(1))==obj.getNrLowLevVars(indexOfLowLev(i)),'The dimension of all lower level variables must be the same,if one input is used for all lower level problems')
    end
end


%%Plot The Problem
%initialize the plot
%get the number of plots in vertical and horizontal direction
nrHorizontal=ceil(sqrt(NrIndexSets));
nrVertical=ceil(NrIndexSets/nrHorizontal);
hf=figure('Visible','off');
set(hf,'Units','centimeter');
oldPos=get(hf,'Position');
height=min(10*nrVertical,maxheight);
width=min(14*nrHorizontal,maxwidth);
set(hf,'Position',[oldPos(1),oldPos(2),width,height]);

%initialize the subplots and save them
handleSubPlots=cell(1,NrIndexSets);
for i=1:NrIndexSets;
    handleSubPlots{i}=subplot(nrVertical,nrHorizontal,i);
    ylabel(handleSubPlots{i},'xx');
    xlabel(handleSubPlots{i},'yy')
    title(handleSubPlots{i},sprintf('yyyyyyyyyyyy'));
end


%compute distance between two plots
if nrVertical>1
    h1=handleSubPlots{1};
    h2=handleSubPlots{nrHorizontal+1};
    set(h1,'Units','centimeter')
    set(h2,'Units','centimeter')
    posh1=get(h1,'Position');
    posh2=get(h2,'Position');
    distVertical=posh1(2)-posh2(2)-posh2(4);
else
    distVertical=0;
end
if nrHorizontal>1
    h1=handleSubPlots{1};
    h2=handleSubPlots{2};
    set(h1,'Units','centimeter')
    set(h2,'Units','centimeter')
    posh1=get(h1,'Position');
    posh2=get(h2,'Position');
    distHorizontal=posh2(1)-posh1(1)-posh1(3);
else
    distHorizontal=0;
end

% %compute the distance to the left, right, top, bottom
% h1=subplot(nrVertical,nrHorizontal,1);
% set(h1,'Units','centimeter');
% 
% h1=subplot(nrVertical,nrHorizontal,nrHorizontal);
% 
% 
%     
%     set(h2,'Units','centimeter')


%compute the original size of a subplot
h1=handleSubPlots{1};
set(h1,'Units','centimeter');
posh1=get(h1,'Position');
originalHeightOfSubPlot=posh1(4);
originalWidthOfSubPlot=posh1(3);

heightsSubPlot=zeros(nrVertical,nrHorizontal);
widthsSubPlot=zeros(nrVertical,nrHorizontal);

for i=1:NrIndexSets
    %getTheSubplot + indices
    h=handleSubPlots{i};
    row = floor( (i-1)/(nrHorizontal) ) + 1;
    col = mod( i-1, nrHorizontal ) + 1;
    %check in which dimension the plot should be done
    n=obj.getNrLowLevVars(indexOfLowLev(i));
    assert(n==1||n==2,'we can only plot problems with a one or a two dimensional indexset')
    
    %extract the points and bounds corresponding to the i-th index set.
    inputPoints=cell(1,2*nrpoints);
    for k=1:nrpoints
        Y=varargin{2*k-1};
        color=varargin{2*k};
        if isa(Y,'cell')
            inputPoints{2*k-1}=Y{i};
        else
            inputPoints{2*k-1}=Y;
        end
        if isa(color,'cell')
            inputPoints{2*k}=color{i};
        else
            inputPoints{2*k}=color;
        end
    end
    lb_i=lb{i};
    ub_i=ub{i};

    switch n
        case 1
            [heightsSubPlot(row,col),widthsSubPlot(row,col)]=obj.plotLowerLevelOfGSIPImplicitly1D(h,indexOfLowLev(i),x,lb_i,ub_i,inputPoints{:}); 
        case 2
            %if nargin>7
            if strcmp(varargin{end},'filled')
                    [heightsSubPlot(row,col),widthsSubPlot(row,col)]=obj.plotLowerLevelOfGSIPImplicitly2D2(h,indexOfLowLev(i),x,lb_i,ub_i,inputPoints{:});
            else
%                     warning('To get a filled picture give as a last input ''filled''');
%                 end
%             else
                [heightsSubPlot(row,col),widthsSubPlot(row,col)]=obj.plotLowerLevelOfGSIPImplicitly2D(h,indexOfLowLev(i),x,lb_i,ub_i,inputPoints{:});
            end
        otherwise
            error('The function is only implemented for one and two dimensional indexsets.');
    end
    
end

%compute the height and width of the Window
maxHeightInRow=max(heightsSubPlot,[],2);
maxWidthInCol=max(widthsSubPlot,[],1);
width =width-sum(originalWidthOfSubPlot-maxWidthInCol);
height=height-sum(originalHeightOfSubPlot-maxHeightInRow);
set(hf,'Position',[0,0,width,height]);

%Adjust the Position of the subplots
h=handleSubPlots{nrHorizontal*(nrVertical-1)+1};
set(h,'Units','centimeter');
lowLeftPosition=get(h,'Position');
actualyPos=lowLeftPosition(2);
actualxPos=lowLeftPosition(1);
for i=nrVertical:-1:1
    for j=1:nrHorizontal
        if  nrHorizontal*(i-1)+j<=NrIndexSets
            h=handleSubPlots{nrHorizontal*(i-1)+j};
            oldPos=get(h,'Position');
            set(h,'Position',[actualxPos,actualyPos,oldPos(3),oldPos(4)]);
            actualxPos=actualxPos+maxWidthInCol(j)+distHorizontal;
        else
        end
        
    end
    actualxPos=lowLeftPosition(1);
    actualyPos=actualyPos+distVertical+maxHeightInRow(i)+1/nrVertical;
end
%Move the window to the center and make it visible
movegui(hf,'center');
set(hf,'Visible','on')
end