classdef ContainerHyperCube < Container
    %C=ContainerHyperCube(n,0) creates an instance of the container C given by:
    %
    %       C={ y in R^n with:
    %               -yi <= 0 for i=1,...,n  
    %               yi-1 <= 0 for i=n+1,...,2n
    %    
    %   Bounding box: [0,1]^n
    %
    %C=ContainerHyperCube(n,1) creates an instance of the container C given
    %as above with additional restriction (2n+1 th)
    %
    %yn-0.5-0.5*sum_i=1^n-1(yi-0.5)^2
    
    
    properties (GetAccess=protected)
        dim                                
        nrOfConstFuns                      
        BoundingBox;
        
        edgeLength;
        concavified
    end
    
    methods
        function p=ContainerHyperCube(dim,concavified)
            p.dim=dim;
            p.nrOfConstFuns=2*dim;
            p.edgeLength=1;
            if concavified
                p.nrOfConstFuns=p.nrOfConstFuns+1;
            end
            p.BoundingBox=[zeros(1,dim);p.edgeLength*ones(1,dim)];
            p.concavified=concavified;

        end
    end
    
    methods
        %container functions
        function g=evalDescrFuns(obj,i,y)   
            if i<=obj.dim
                g=-y(:,i);
            elseif i>obj.dim && obj.dim*2>=i
                g=y(:,i-obj.dim)-obj.edgeLength;
                
            else
                l=obj.edgeLength;
                g=y(:,end)-2/l*((y(:,1:end-1)-l/2).^2*ones(obj.dim-1,1))-l/2;                
            end
               
        end
        %derivative
        function Dy_g=DyDescrFuns(obj,i,y)  
            m=size(y,1);
            
            if i<=obj.dim
                Dy_g=zeros(m,obj.dim);
                Dy_g(:,i)=-ones(m,1);
            elseif i>obj.dim && obj.dim*2>=i
                Dy_g=zeros(m,obj.dim);
                Dy_g(:,i-obj.dim)=ones(m,1);
            else
                l=obj.edgeLength;
                Dy_g=[-4/l*(y(:,1:end-1)-l/2),ones(m,1)];
            end
           
        end
    end
    
    methods
        function plotContainer(obj)
            switch obj.dim
                case 2
                    l=obj.edgeLength;
                    x1=(0:l*0.05:l)';
                    if obj.concavified
                        y1=2/l*(x1-l/2).^2+l/2;
                    else
                        y1=l*ones(size(x1));
                    end

                    X = [0; ... %vertex
                         0; ...%vertex
                         x1;... %parabola
                        l]; %vertex
                    Y = [0; ...        %vertex
                         l; ...%vertex
                        y1;... %parabola
                        0];     %vertex

                    ph = fill(X, Y, [135/255, 206/255, 250/255]); % sky blue
                    set(ph, 'EdgeColor', 'blue');
                    hold on;
                case 3
                    l=obj.edgeLength;
                    fill3([0,0,l,l],[0,l,l,0],[0,0,0,0],'blue', 'facealpha', 0.25);
                    hold on
                    fill3([0,0,l,l],[0,0,0,0],[0,l,l,0],'blue', 'facealpha', 0.25);
                    fill3([0,0,0,0],[0,l,l,0],[0,0,l,l],'blue', 'facealpha', 0.25);


                    fill3([0,0,l,l],[l,l,l,l],[0,l,l,0],'blue', 'facealpha', 0.25);
                    fill3([l,l,l,l],[0,l,l,0],[0,0,l,l],'blue', 'facealpha', 0.25);

                    if obj.concavified
                        %Bestimmen der oberen Fl�che
                        x=(0:l*0.05:l);
                        y=(0:l*0.05:l);
                        [X,Y]=meshgrid(x,y);
                        Zt=2/l*((X-l/2).^2+(Y-l/2).^2)+l/2;
                        Z=min(l*ones(size(Zt)),Zt);
                        s=surf(X,Y,Z);
                        set(s,'FaceColor', 'blue', 'FaceAlpha', 0.25);
                    else
                    fill3([0,0,l,l],[0,l,l,0],[l,l,l,l],'blue', 'facealpha', 0.25);
                    end
                    hold on;
            end
        end
    end    

    
    
end
